@@ -1,7 +1,13 @@
 /*
  * ==========================================================================
- *                  NetPerfMeter -- Network Performance Meter                 
- *                 Copyright (C) 2009-2024 by Thomas Dreibholz
+ *         _   _      _   ____            __ __  __      _
+ *        | \ | | ___| |_|  _ \ ___ _ __ / _|  \/  | ___| |_ ___ _ __
+ *        |  \| |/ _ \ __| |_) / _ \ '__| |_| |\/| |/ _ \ __/ _ \ '__|
+ *        | |\  |  __/ |_|  __/  __/ |  |  _| |  | |  __/ ||  __/ |
+ *        |_| \_|\___|\__|_|   \___|_|  |_| |_|  |_|\___|\__\___|_|
+ *
+ *                  NetPerfMeter -- Network Performance Meter
+ *                 Copyright (C) 2009-2026 by Thomas Dreibholz
  * ==========================================================================
  *
  * This program is free software: you can redistribute it and/or modify
@@ -17,7 +23,7 @@
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
- * Contact:  thomas.dreibholz@gmail.com
+ * Contact:  dreibh@simula.no
  * Homepage: https://www.nntb.no/~dreibh/netperfmeter/
  */
 
@@ -41,21 +47,21 @@ class CPUStatus
    void update();
 
    inline unsigned int getNumberOfCPUs() const {
-      return(CPUs);
+      return CPUs;
    }
    inline unsigned int getCpuStates() const {
-      return(CpuStates);
+      return CpuStates;
    }
    inline const char* getCpuStateName(const unsigned int index) const {
       assert(index < CpuStates);
-      return(CpuStateNames[index]);
+      return CpuStateNames[index];
    }
    inline float getCpuStatePercentage(const unsigned int cpuIndex,
                                       const unsigned int stateIndex) const {
       assert(cpuIndex <= CPUs);
       assert(stateIndex < CpuStates);
       const unsigned int index = (cpuIndex * CpuStates) + stateIndex;
-      return(Percentages[index]);
+      return Percentages[index];
    }
    float getCpuUtilization(const unsigned int cpuIndex) const;
 
