@@ -1,7 +1,13 @@
 /*
  * ==========================================================================
+ *         _   _      _   ____            __ __  __      _
+ *        | \ | | ___| |_|  _ \ ___ _ __ / _|  \/  | ___| |_ ___ _ __
+ *        |  \| |/ _ \ __| |_) / _ \ '__| |_| |\/| |/ _ \ __/ _ \ '__|
+ *        | |\  |  __/ |_|  __/  __/ |  |  _| |  | |  __/ ||  __/ |
+ *        |_| \_|\___|\__|_|   \___|_|  |_| |_|  |_|\___|\__\___|_|
+ *
  *                  NetPerfMeter -- Network Performance Meter
- *                 Copyright (C) 2009-2024 by Thomas Dreibholz
+ *                 Copyright (C) 2009-2026 by Thomas Dreibholz
  * ==========================================================================
  *
  * This program is free software: you can redistribute it and/or modify
@@ -17,15 +23,15 @@
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
- * Contact:  thomas.dreibholz@gmail.com
+ * Contact:  dreibh@simula.no
  * Homepage: https://www.nntb.no/~dreibh/netperfmeter/
  */
 
+#include "assure.h"
 #include "defragmenter.h"
 
 #include <stdio.h>
 #include <stdlib.h>
-#include <assert.h>
 #include <map>
 
 
@@ -63,13 +69,13 @@ Defragmenter::~Defragmenter()
 // ###### Print FragmentSet #################################################
 void Defragmenter::print(std::ostream& os)
 {
-   os << "FragmentSet:" << std::endl;
+   os << "FragmentSet:\n";
    for(std::map<uint32_t, Frame*>::iterator frameIterator = FrameSet.begin();
          frameIterator != FrameSet.end(); frameIterator++) {
       Frame* frame = frameIterator->second;
       os << "   - Frame " << frame->FrameID
          << ", LastUpdate=" << frame->LastUpdate
-         << ":" << std::endl;
+         << ":\n";
 
       for(std::map<uint64_t, Fragment*>::iterator fragmentIterator = frame->FragmentSet.begin();
           fragmentIterator != frame->FragmentSet.end(); fragmentIterator++) {
@@ -83,7 +89,7 @@ void Defragmenter::print(std::ostream& os)
          if(fragment->Flags & NPMDF_FRAME_END) {
             os << "<End> ";
          }
-         os << std::endl;
+         os << "\n";
       }
    }
 }
@@ -140,13 +146,13 @@ bool Defragmenter::getFirstFragment(Frame*&    frame,
    if(FrameIterator != FrameSet.end()) {
       frame            = FrameIterator->second;
       FragmentIterator = frame->FragmentSet.begin();
-      assert(FragmentIterator != frame->FragmentSet.end());
+      assure(FragmentIterator != frame->FragmentSet.end());
       fragment = FragmentIterator->second;
-      return(true);
+      return true;
    }
-   frame    = NULL;
-   fragment = NULL;
-   return(false);
+   frame    = nullptr;
+   fragment = nullptr;
+   return false;
 }
 
 
@@ -164,13 +170,13 @@ bool Defragmenter::getNextFragment(Frame*&    frame,
    if(FragmentIterator == frame->FragmentSet.end()) {
       FrameIterator++;
       if(FrameIterator == FrameSet.end()) {
-         frame    = NULL;
-         fragment = NULL;
+         frame    = nullptr;
+         fragment = nullptr;
       }
       else {
          frame            = FrameIterator->second;
          FragmentIterator = frame->FragmentSet.begin();
-         assert(FragmentIterator != frame->FragmentSet.end());
+         assure(FragmentIterator != frame->FragmentSet.end());
          fragment = FragmentIterator->second;
       }
    }
@@ -184,10 +190,16 @@ bool Defragmenter::getNextFragment(Frame*&    frame,
       if(lastFrame->FragmentSet.size() == 0) {
          FrameSet.erase(lastFrameIterator);
          delete lastFrame;
+         if(lastFrame == frame) {   // Ensure reference gets invalidated!
+            frame = nullptr;
+         }
+      }
+      if(lastFragment == fragment) {   // Ensure reference gets invalidated!
+         fragment = nullptr;
       }
    }
 
-   return(fragment != NULL);
+   return ((frame != nullptr) && (fragment != nullptr));
 }
 
 
