@@ -1,7 +1,13 @@
 /*
  * ==========================================================================
+ *         _   _      _   ____            __ __  __      _
+ *        | \ | | ___| |_|  _ \ ___ _ __ / _|  \/  | ___| |_ ___ _ __
+ *        |  \| |/ _ \ __| |_) / _ \ '__| |_| |\/| |/ _ \ __/ _ \ '__|
+ *        | |\  |  __/ |_|  __/  __/ |  |  _| |  | |  __/ ||  __/ |
+ *        |_| \_|\___|\__|_|   \___|_|  |_| |_|  |_|\___|\__\___|_|
+ *
  *                  NetPerfMeter -- Network Performance Meter
- *                 Copyright (C) 2009-2024 by Thomas Dreibholz
+ *                 Copyright (C) 2009-2026 by Thomas Dreibholz
  * ==========================================================================
  *
  * This program is free software: you can redistribute it and/or modify
@@ -17,151 +23,30 @@
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
- * Contact:  thomas.dreibholz@gmail.com
+ * Contact:  dreibh@simula.no
  * Homepage: https://www.nntb.no/~dreibh/netperfmeter/
  */
 
 #ifndef FLOW_H
 #define FLOW_H
 
-#include "thread.h"
+#include "cpustatus.h"
+#include "defragmenter.h"
+#include "flowbandwidthstats.h"
+#include "flowmanager.h"
+#include "flowtrafficspec.h"
+#include "measurement.h"
 #include "messagereader.h"
 #include "outputfile.h"
-#include "flowbandwidthstats.h"
-#include "flowtrafficspec.h"
-#include "defragmenter.h"
-#include "measurement.h"
-#include "cpustatus.h"
+#include "thread.h"
 #include "tools.h"
 
-#include <poll.h>
+// #include <poll.h>
 
 #include <vector>
 #include <map>
 
 
-class Flow;
-
-class FlowManager : public Thread
-{
-   friend class Flow;
-
-   // ====== Methods ========================================================
-   protected:
-   FlowManager();
-   virtual ~FlowManager();
-
-   public:
-   inline static FlowManager* getFlowManager() {
-      return(&FlowManagerSingleton);
-   }
-   inline MessageReader* getMessageReader() {
-      return(&Reader);
-   }
-   inline std::vector<Flow*>& getFlowSet() {   // Internal usage only!
-      return(FlowSet);
-   }
-
-   inline void enableDisplay() {
-      lock();
-      DisplayOn = true;
-      unlock();
-   }
-   inline void disableDisplay() {
-      lock();
-      DisplayOn = false;
-      unlock();
-   }
-
-   void addSocket(const int protocol, const int socketDescriptor);
-   Flow* identifySocket(const uint64_t         measurementID,
-                        const uint32_t         flowID,
-                        const uint16_t         streamID,
-                        const int              socketDescriptor,
-                        const sockaddr_union*  from,
-                        const OutputFileFormat vectorFileFormat,
-                        int&                   controlSocketDescriptor);
-   void removeSocket(const int  socketDescriptor,
-                     const bool closeSocket = true);
-
-   void addFlow(Flow* flow);
-   void removeFlow(Flow* flow);
-   void printFlows(std::ostream& os,
-                   const bool    printStatistics);
-
-   bool startMeasurement(const uint64_t           measurementID,
-                         const unsigned long long now,
-                         const char*              vectorNamePattern,
-                         const OutputFileFormat   vectorFileFormat,
-                         const char*              scalarNamePattern,
-                         const OutputFileFormat   scalarFileFormat,
-                         const bool               printFlows = false);
-   void stopMeasurement(const uint64_t            measurementID,
-                        const bool                printFlows = false,
-                        const unsigned long long  now        = getMicroTime());
-
-   void writeScalarStatistics(const uint64_t           measurementID,
-                              const unsigned long long now,
-                              OutputFile&              scalarFile,
-                              const unsigned long long firstStatisticsEvent);
-   void writeVectorStatistics(const uint64_t           measurementID,
-                              const unsigned long long now,
-                              OutputFile&              vectorFile,
-                              FlowBandwidthStats&      globalStats,
-                              FlowBandwidthStats&      relGlobalStats,
-                              const unsigned long long firstStatisticsEvent,
-                              const unsigned long long lastStatisticsEvent);
-
-   Flow* findFlow(const uint64_t measurementID,
-                  const uint32_t flowID,
-                  const uint16_t streamID);
-   Flow* findFlow(const int socketDescriptor,
-                  uint16_t  streamID);
-   Flow* findFlow(const struct sockaddr* from);
-
-
-   bool addMeasurement(Measurement* measurement);
-   Measurement* findMeasurement(const uint64_t measurementID);
-   void printMeasurements(std::ostream& os);
-   void removeMeasurement(Measurement* measurement);
-
-
-   // ====== Protected Methods ==============================================
-   protected:
-   void run();
-
-
-   // ====== Private Methods ================================================
-   unsigned long long getNextEvent();
-   void handleEvents(const unsigned long long now);
-
-
-   // ====== Private Data ===================================================
-   private:
-   static FlowManager FlowManagerSingleton;
-
-   // ------ Flow Management ------------------------------------------------
-   MessageReader      Reader;
-   std::vector<Flow*> FlowSet;
-   std::map<int, int> UnidentifiedSockets;
-   bool               UpdatedUnidentifiedSockets;
-   bool               DisplayOn;
-   FlowBandwidthStats CurrentGlobalStats;
-   FlowBandwidthStats LastGlobalStats;
-
-   // ------ Measurement Management -----------------------------------------
-   std::map<uint64_t, Measurement*> MeasurementSet;
-   unsigned long long               DisplayInterval;
-   unsigned long long               FirstDisplayEvent;
-   unsigned long long               LastDisplayEvent;
-   unsigned long long               NextDisplayEvent;
-   CPUStatus                        CPULoadStats;
-   FlowBandwidthStats               GlobalStats;      // For displaying only
-   FlowBandwidthStats               RelGlobalStats;   // For displaying only
-   CPUStatus                        CPUDisplayStats;  // For displaying only
-};
-
-
 class Flow : public Thread
 {
    public:
@@ -178,39 +63,43 @@ class Flow : public Thread
         const uint32_t         flowID,
         const uint16_t         streamID,
         const FlowTrafficSpec& trafficSpec,
-        const int              controlSocketDescriptor = -1);
+        const int              controlSocket = -1);
    virtual ~Flow();
 
 
    inline uint64_t getMeasurementID() const {
-      return(MeasurementID);
+      return MeasurementID;
    }
    inline uint32_t getFlowID() const {
-      return(FlowID);
+      return FlowID;
    }
    inline uint16_t getStreamID() const {
-      return(StreamID);
+      return StreamID;
    }
    inline int getSocketDescriptor() const {
-      return(SocketDescriptor);
+      return SocketDescriptor;
    }
+   void setSocketDescriptor(const int  socketDescriptor,
+                            const bool originalSocketDescriptor = true,
+                            const bool deleteWhenFinished       = true);
+
    inline Defragmenter* getDefragmenter() {
-      return(&MyDefragmenter);
+      return &MyDefragmenter;
    }
-   inline int getRemoteControlSocketDescriptor() const {
-      return(RemoteControlSocketDescriptor);
+   inline int getControlSocketDescriptor() const {
+      return ControlSocketDescriptor;
    }
    inline const FlowTrafficSpec& getTrafficSpec() const {
-      return(TrafficSpec);
+      return TrafficSpec;
    }
    inline FlowStatus getOutputStatus() const {
-      return(OutputStatus);
+      return OutputStatus;
    }
    inline FlowStatus getInputStatus() const {
-      return(InputStatus);
+      return InputStatus;
    }
    inline bool isAcceptedIncomingFlow() const {
-      return(RemoteControlSocketDescriptor != -1);
+      return ControlSocketDescriptor != -1;
    }
 
    inline void endOfInput() {
@@ -218,65 +107,61 @@ class Flow : public Thread
    }
 
    inline const FlowBandwidthStats& getCurrentBandwidthStats() const {
-      return(CurrentBandwidthStats);
+      return CurrentBandwidthStats;
    }
    inline FlowBandwidthStats& getCurrentBandwidthStats() {
-      return(CurrentBandwidthStats);
+      return CurrentBandwidthStats;
    }
    inline unsigned long long getFirstTransmission() const {
-      return(FirstTransmission);
+      return FirstTransmission;
    }
    inline unsigned long long getLastTransmission() const {
-      return(LastTransmission);
+      return LastTransmission;
    }
    inline unsigned long long getFirstReception() const {
-      return(FirstReception);
+      return FirstReception;
    }
    inline unsigned long long getLastReception() const {
-      return(LastReception);
+      return LastReception;
    }
 
    inline uint32_t nextOutboundFrameID() {
-      return(++LastOutboundFrameID);
+      return ++LastOutboundFrameID;
    }
    inline uint64_t nextOutboundSeqNumber() {
-      return(++LastOutboundSeqNumber);
+      return ++LastOutboundSeqNumber;
    }
 
    inline bool isRemoteAddressValid() const {
-      return(RemoteAddressIsValid);
+      return RemoteAddressIsValid;
    }
    inline const sockaddr* getRemoteAddress() const {
-      return(&RemoteAddress.sa);
+      return &RemoteAddress.sa;
    }
 
    inline OutputFile& getVectorFile() {
-      return(VectorFile);
+      return VectorFile;
    }
    inline const OutputFile& getVectorFile() const {
-      return(VectorFile);
+      return VectorFile;
    }
    inline double getJitter() const {
-      return(Jitter);
+      return Jitter;
    }
    inline void setJitter(const double jitter) {
       Jitter = jitter;
    }
    inline double getDelay() const {
-      return(Delay);
+      return Delay;
    }
    inline void setDelay(const double transitTime) {
       Delay = transitTime;
    }
 
    inline Measurement* getMeasurement() const {
-      return(MyMeasurement);
-   }
-   inline void setMeasurement(Measurement* measurement) {
-      lock();
-      MyMeasurement = measurement;
-      unlock();
+      return MyMeasurement;
    }
+   bool setMeasurement(Measurement* measurement);
 
    inline static std::string getNodeOutputName(const std::string& pattern,
                                                const char*        type,
@@ -286,7 +171,7 @@ class Flow : public Thread
       dissectName(pattern, prefix, suffix);
       const std::string result = prefix + "-" + std::string(type) +
                                  std::string(extension) + suffix;
-      return(result);
+      return result;
    }
 
    bool initializeVectorFile(const char* name, const OutputFileFormat format);
@@ -310,9 +195,7 @@ class Flow : public Thread
    void resetStatistics();
 
    bool configureSocket(const int socketDescriptor);
-   void setSocketDescriptor(const int  socketDescriptor,
-                            const bool originalSocketDescriptor = true,
-                            const bool deleteWhenFinished       = true);
+
    bool activate();
    void deactivate(const bool asyncStop = false);
 
@@ -338,7 +221,7 @@ class Flow : public Thread
    bool               DeleteWhenFinished;
    pollfd*            PollFDEntry;   // For internal usage by FlowManager
 
-   int                RemoteControlSocketDescriptor;
+   int                ControlSocketDescriptor;
    sockaddr_union     RemoteAddress;
    bool               RemoteAddressIsValid;
 
