@@ -1,7 +1,13 @@
 /*
  * ==========================================================================
+ *         _   _      _   ____            __ __  __      _
+ *        | \ | | ___| |_|  _ \ ___ _ __ / _|  \/  | ___| |_ ___ _ __
+ *        |  \| |/ _ \ __| |_) / _ \ '__| |_| |\/| |/ _ \ __/ _ \ '__|
+ *        | |\  |  __/ |_|  __/  __/ |  |  _| |  | |  __/ ||  __/ |
+ *        |_| \_|\___|\__|_|   \___|_|  |_| |_|  |_|\___|\__\___|_|
+ *
  *                  NetPerfMeter -- Network Performance Meter
- *                 Copyright (C) 2009-2024 by Thomas Dreibholz
+ *                 Copyright (C) 2009-2026 by Thomas Dreibholz
  * ==========================================================================
  *
  * This program is free software: you can redistribute it and/or modify
@@ -17,23 +23,24 @@
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
- * Contact:  thomas.dreibholz@gmail.com
+ * Contact:  dreibh@simula.no
  * Homepage: https://www.nntb.no/~dreibh/netperfmeter/
  */
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <time.h>
-#include <sys/time.h>
 
 
 // ###### Get current time stamp ############################################
-unsigned long long getMicroTime()
+static unsigned long long getMicroTime()
 {
-  struct timeval tv;
-  gettimeofday(&tv,NULL);
-  return(((unsigned long long)tv.tv_sec * (unsigned long long)1000000) +
-         (unsigned long long)tv.tv_usec);
+  timespec ts;
+  if(__builtin_expect( (clock_gettime(CLOCK_REALTIME, &ts) != 0) , 0)) {
+     perror("clock_gettime():");
+     abort();
+  }
+  return ((unsigned long long)ts.tv_sec * 1000000ULL) + (ts.tv_nsec / 1000);
 }
 
 
