@@ -1,7 +1,13 @@
 /*
  * ==========================================================================
- *                  NetPerfMeter -- Network Performance Meter                 
- *                 Copyright (C) 2009-2024 by Thomas Dreibholz
+ *         _   _      _   ____            __ __  __      _
+ *        | \ | | ___| |_|  _ \ ___ _ __ / _|  \/  | ___| |_ ___ _ __
+ *        |  \| |/ _ \ __| |_) / _ \ '__| |_| |\/| |/ _ \ __/ _ \ '__|
+ *        | |\  |  __/ |_|  __/  __/ |  |  _| |  | |  __/ ||  __/ |
+ *        |_| \_|\___|\__|_|   \___|_|  |_| |_|  |_|\___|\__\___|_|
+ *
+ *                  NetPerfMeter -- Network Performance Meter
+ *                 Copyright (C) 2009-2026 by Thomas Dreibholz
  * ==========================================================================
  *
  * This program is free software: you can redistribute it and/or modify
@@ -17,7 +23,7 @@
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
- * Contact:  thomas.dreibholz@gmail.com
+ * Contact:  dreibh@simula.no
  * Homepage: https://www.nntb.no/~dreibh/netperfmeter/
  */
 
@@ -50,43 +56,45 @@ Measurement::~Measurement()
 
 // ###### Initialize measurement ############################################
 bool Measurement::initialize(const unsigned long long now,
+                             const int                controlSocketDescriptor,
                              const uint64_t           measurementID,
                              const char*              vectorNamePattern,
                              const OutputFileFormat   vectorFileFormat,
                              const char*              scalarNamePattern,
                              const OutputFileFormat   scalarFileFormat)
 {
-   MeasurementID        = measurementID;
-   FirstStatisticsEvent = 0;
-   LastStatisticsEvent  = 0;
-   NextStatisticsEvent  = 0;
+   ControlSocketDescriptor = controlSocketDescriptor;
+   MeasurementID           = measurementID;
+   FirstStatisticsEvent    = 0;
+   LastStatisticsEvent     = 0;
+   NextStatisticsEvent     = 0;
 
-   if(FlowManager::getFlowManager()->addMeasurement(this)) {
-      VectorNamePattern = (vectorNamePattern != NULL) ?
+   if(FlowManager::getFlowManager()->addMeasurement(ControlSocketDescriptor, this)) {
+      VectorNamePattern = (vectorNamePattern != nullptr) ?
                              std::string(vectorNamePattern) : std::string();
       const bool s1 = VectorFile.initialize(
-                         (vectorNamePattern != NULL) ?
-                            Flow::getNodeOutputName(vectorNamePattern, "active").c_str() : NULL,
+                         (vectorNamePattern != nullptr) ?
+                            Flow::getNodeOutputName(vectorNamePattern, "active").c_str() : nullptr,
                          vectorFileFormat);
-      ScalarNamePattern = (scalarNamePattern != NULL) ?
+      ScalarNamePattern = (scalarNamePattern != nullptr) ?
                              std::string(scalarNamePattern) : std::string();
       const bool s2 = ScalarFile.initialize(
-                         (scalarNamePattern != NULL) ?
-                            Flow::getNodeOutputName(scalarNamePattern, "active").c_str() : NULL,
+                         (scalarNamePattern != nullptr) ?
+                            Flow::getNodeOutputName(scalarNamePattern, "active").c_str() : nullptr,
                          scalarFileFormat);
-      return(s1 && s2);
+      return s1 && s2;
    }
-   return(false);
+   return false;
 }
 
 
 // ###### Finish measurement ################################################
 bool Measurement::finish(const bool closeFiles)
 {
-   FlowManager::getFlowManager()->removeMeasurement(this);
+   FlowManager::getFlowManager()->removeMeasurement(ControlSocketDescriptor, this);
    const bool s1 = VectorFile.finish(closeFiles);
    const bool s2 = ScalarFile.finish(closeFiles);
-   return(s1 && s2);
+   return s1 && s2;
 }
 
 
