@@ -1,7 +1,13 @@
 /*
  * ==========================================================================
- *                  NetPerfMeter -- Network Performance Meter                 
- *                 Copyright (C) 2009-2024 by Thomas Dreibholz
+ *         _   _      _   ____            __ __  __      _
+ *        | \ | | ___| |_|  _ \ ___ _ __ / _|  \/  | ___| |_ ___ _ __
+ *        |  \| |/ _ \ __| |_) / _ \ '__| |_| |\/| |/ _ \ __/ _ \ '__|
+ *        | |\  |  __/ |_|  __/  __/ |  |  _| |  | |  __/ ||  __/ |
+ *        |_| \_|\___|\__|_|   \___|_|  |_| |_|  |_|\___|\__\___|_|
+ *
+ *                  NetPerfMeter -- Network Performance Meter
+ *                 Copyright (C) 2009-2026 by Thomas Dreibholz
  * ==========================================================================
  *
  * This program is free software: you can redistribute it and/or modify
@@ -17,7 +23,7 @@
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
- * Contact:  thomas.dreibholz@gmail.com
+ * Contact:  dreibh@simula.no
  * Homepage: https://www.nntb.no/~dreibh/netperfmeter/
  */
 
@@ -30,8 +36,8 @@
 // ###### Constructor #######################################################
 OutputFile::OutputFile()
 {
-   File       = NULL;
-   BZFile     = NULL;
+   File       = nullptr;
+   BZFile     = nullptr;
    WriteError = false;
    Line       = 0;
 }
@@ -54,7 +60,7 @@ bool OutputFile::initialize(const char*            name,
 
    Line   = 0;
    Format = format;
-   if(name != NULL) {
+   if(name != nullptr) {
       Name = std::string(name);
    }
    else {
@@ -62,15 +68,14 @@ bool OutputFile::initialize(const char*            name,
    }
 
    // ====== Initialize file ================================================
-   File   = NULL;
-   BZFile = NULL;
+   File   = nullptr;
+   BZFile = nullptr;
    if(format != OFF_None) {
-      File = (name != NULL) ? fopen(name, "w+") : tmpfile();
-      if(File == NULL) {
-         std::cerr << "ERROR: Unable to create output file <"
-                   << Name << ">!" << std::endl;
+      File = (name != nullptr) ? fopen(name, "w+") : tmpfile();
+      if(File == nullptr) {
+         std::cerr << "ERROR: Unable to create output file <" << Name << ">!\n";
          WriteError = true;
-         return(false);
+         return false;
       }
 
       // ====== Initialize BZip2 compressor ====================================
@@ -78,18 +83,17 @@ bool OutputFile::initialize(const char*            name,
          int bzerror;
          BZFile = BZ2_bzWriteOpen(&bzerror, File, compressionLevel, 0, 30);
          if(bzerror != BZ_OK) {
-            std::cerr << "ERROR: Unable to initialize BZip2 compression on file <"
-                      << Name << ">!" << std::endl
-                      << "Reason: " << BZ2_bzerror(BZFile, &bzerror) << std::endl;
-            BZ2_bzWriteClose(&bzerror, BZFile, 0, NULL, NULL);
+            std::cerr << "ERROR: Unable to initialize BZip2 compression on file <" << Name << ">!\n"
+                      << "Reason: " << BZ2_bzerror(BZFile, &bzerror) << "\n";
+            BZ2_bzWriteClose(&bzerror, BZFile, 0, nullptr, nullptr);
             WriteError = true;
             finish();
-            return(false);
+            return false;
          }
       }
       WriteError = false;
    }
-   return(true);
+   return true;
 }
 
 
@@ -112,12 +116,11 @@ bool OutputFile::finish(const bool          closeFile,
          }
       }
       else {
-         std::cerr << "ERROR: Unable to finish BZip2 compression on file <"
-                   << Name << ">!" << std::endl
-                   << "Reason: " << BZ2_bzerror(BZFile, &bzerror) << std::endl;
+         std::cerr << "ERROR: Unable to finish BZip2 compression on file <" << Name << ">!\n"
+                   << "Reason: " << BZ2_bzerror(BZFile, &bzerror) << "\n";
          WriteError = true;
       }
-      BZFile = NULL;
+      BZFile = nullptr;
    }
    else {
       if(bytesIn) {
@@ -132,11 +135,10 @@ bool OutputFile::finish(const bool          closeFile,
    if(File) {
       if(closeFile) {
          if(fclose(File) != 0) {
-            std::cerr << "ERROR: Unable to close output file <"
-                      << Name << ">!" << std::endl;
+            std::cerr << "ERROR: Unable to close output file <" << Name << ">!\n";
             WriteError = true;
          }
-         File = NULL;
+         File = nullptr;
       }
       else {
          rewind(File);
@@ -147,7 +149,7 @@ bool OutputFile::finish(const bool          closeFile,
    if(WriteError) {
       unlink(Name.c_str());
    }
-   return(!WriteError);
+   return !WriteError;
 }
 
 
@@ -163,9 +165,9 @@ bool OutputFile::printf(const char* str, ...)
       buffer[sizeof(buffer) - 1] = 0x00;   // Just to be really sure ...
       va_end(va);
 
-      return(write((const char*)&buffer, bufferLength));
+      return write((const char*)&buffer, bufferLength);
    }
-   return(true);
+   return true;
 }
 
 
@@ -178,22 +180,19 @@ bool OutputFile::write(const char* buffer, const size_t bufferLength)
          int bzerror;
          BZ2_bzWrite(&bzerror, BZFile, (void*)buffer, bufferLength);
          if(bzerror != BZ_OK) {
-            std::cerr << std::endl
-                      << "ERROR: libbz2 failed to write into file <"
-                      << Name << ">!" << std::endl
-                      << "Reason: " << BZ2_bzerror(BZFile, &bzerror) << std::endl;
-            return(false);
+            std::cerr << "\nERROR: libbz2 failed to write into file <" << Name << ">!\n"
+                      << "Reason: " << BZ2_bzerror(BZFile, &bzerror) << "\n";
+            return false;
          }
       }
 
       // ====== Write string as plain text ==================================
       else if(File) {
          if(fwrite(buffer, bufferLength, 1, File) != 1) {
-            std::cerr << "ERROR: Failed to write into file <"
-                     << Name << ">!" << std::endl;
-            return(false);
+            std::cerr << "ERROR: Failed to write into file <" << Name << ">!\n";
+            return false;
          }
       }
    }
-   return(true);
+   return true;
 }
