@@ -1,7 +1,13 @@
 /*
  * ==========================================================================
+ *         _   _      _   ____            __ __  __      _
+ *        | \ | | ___| |_|  _ \ ___ _ __ / _|  \/  | ___| |_ ___ _ __
+ *        |  \| |/ _ \ __| |_) / _ \ '__| |_| |\/| |/ _ \ __/ _ \ '__|
+ *        | |\  |  __/ |_|  __/  __/ |  |  _| |  | |  __/ ||  __/ |
+ *        |_| \_|\___|\__|_|   \___|_|  |_| |_|  |_|\___|\__\___|_|
+ *
  *                  NetPerfMeter -- Network Performance Meter
- *                 Copyright (C) 2009-2024 by Thomas Dreibholz
+ *                 Copyright (C) 2009-2026 by Thomas Dreibholz
  * ==========================================================================
  *
  * This program is free software: you can redistribute it and/or modify
@@ -17,7 +23,7 @@
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
- * Contact:  thomas.dreibholz@gmail.com
+ * Contact:  dreibh@simula.no
  * Homepage: https://www.nntb.no/~dreibh/netperfmeter/
  */
 
@@ -30,10 +36,12 @@
 // ###### Get current time stamp ############################################
 static unsigned long long getMicroTime()
 {
-  struct timeval tv;
-  gettimeofday(&tv,NULL);
-  return(((unsigned long long)tv.tv_sec * (unsigned long long)1000000) +
-         (unsigned long long)tv.tv_usec);
+  timespec ts;
+  if(__builtin_expect( (clock_gettime(CLOCK_REALTIME, &ts) != 0) , 0)) {
+     perror("clock_gettime():");
+     abort();
+  }
+  return ((unsigned long long)ts.tv_sec * 1000000ULL) + (ts.tv_nsec / 1000);
 }
 
 
@@ -41,7 +49,7 @@ static unsigned long long getMicroTime()
 int main(int argc, char** argv)
 {
    if(argc < 4) {
-      fprintf(stderr, "Usage: %s [Start File] [Total Runs] [Current Run]\n",
+      fprintf(stderr, "Usage: %s start_file total_runs current_run\n",
               argv[0]);
    }
 
@@ -59,13 +67,14 @@ int main(int argc, char** argv)
 
    if(currentRun == 0) {
       FILE* fh = fopen(argv[1], "w");
-      if(fh == NULL) {
+      if(fh == nullptr) {
          fprintf(stderr, "ERROR: Unable to create file <%s>!\n", argv[1]);
          exit(1);
       }
       unsigned long long now = getMicroTime();
       if(fwrite((void*)&now, sizeof(now), 1, fh) != 1) {
          fprintf(stderr, "ERROR: Unable to write timestamp to file <%s>!\n", argv[1]);
+         fclose(fh);
          exit(1);
       }
       fclose(fh);
@@ -74,12 +83,13 @@ int main(int argc, char** argv)
    else {
       unsigned long long startTimeStamp;
       FILE* fh = fopen(argv[1], "r");
-      if(fh == NULL) {
+      if(fh == nullptr) {
          fprintf(stderr, "ERROR: Unable to open file <%s>!\n", argv[1]);
          exit(1);
       }
       if(fread((void*)&startTimeStamp, sizeof(startTimeStamp), 1, fh) != 1) {
          fprintf(stderr, "ERROR: Unable to read timestamp from file <%s>!\n", argv[1]);
+         fclose(fh);
          exit(1);
       }
       fclose(fh);
