@@ -1,7 +1,13 @@
 /*
  * ==========================================================================
- *                  NetPerfMeter -- Network Performance Meter                 
- *                 Copyright (C) 2009-2024 by Thomas Dreibholz
+ *         _   _      _   ____            __ __  __      _
+ *        | \ | | ___| |_|  _ \ ___ _ __ / _|  \/  | ___| |_ ___ _ __
+ *        |  \| |/ _ \ __| |_) / _ \ '__| |_| |\/| |/ _ \ __/ _ \ '__|
+ *        | |\  |  __/ |_|  __/  __/ |  |  _| |  | |  __/ ||  __/ |
+ *        |_| \_|\___|\__|_|   \___|_|  |_| |_|  |_|\___|\__\___|_|
+ *
+ *                  NetPerfMeter -- Network Performance Meter
+ *                 Copyright (C) 2009-2026 by Thomas Dreibholz
  * ==========================================================================
  *
  * This program is free software: you can redistribute it and/or modify
@@ -17,16 +23,14 @@
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
- * Contact:  thomas.dreibholz@gmail.com
+ * Contact:  dreibh@simula.no
  * Homepage: https://www.nntb.no/~dreibh/netperfmeter/
  */
 
-#include <stdio.h>
-#include <unistd.h>
-#include <pthread.h>
-#include <iostream>
-
 #include "thread.h"
+#include "loglevel.h"
+
+#include <pthread.h>
 
 
 // ###### Constructor #######################################################
@@ -53,7 +57,7 @@ void* Thread::startRoutine(void* object)
 {
    Thread* thread = (Thread*)object;
    thread->run();
-   return(NULL);
+   return nullptr;
 }
 
 
@@ -64,16 +68,20 @@ bool Thread::start()
       lock();
       Stopping = false;
       unlock();
-      if(pthread_create(&MyThread, NULL, startRoutine, (void*)this) == 0) {
-         return(true);
+      if(pthread_create(&MyThread, nullptr, startRoutine, (void*)this) == 0) {
+         return true;
       }
       MyThread = 0;
-      std::cerr << "ERROR: Unable to start new thread!" << std::endl;
+      LOG_ERROR
+      stdlog << "Unable to start new thread!" << "\n";
+      LOG_END
    }
    else {
-      std::cerr << "ERROR: Thread already running!" << std::endl;
+      LOG_ERROR
+      stdlog << "Thread is already running!" << "\n";
+      LOG_END
    }
-   return(false);
+   return false;
 }
 
 
@@ -90,7 +98,7 @@ void Thread::stop()
 void Thread::waitForFinish()
 {
    if(MyThread != 0) {
-      pthread_join(MyThread, NULL);
+      pthread_join(MyThread, nullptr);
       MyThread = 0;
    }
 }
