@@ -1,7 +1,13 @@
 /*
  * ==========================================================================
+ *         _   _      _   ____            __ __  __      _
+ *        | \ | | ___| |_|  _ \ ___ _ __ / _|  \/  | ___| |_ ___ _ __
+ *        |  \| |/ _ \ __| |_) / _ \ '__| |_| |\/| |/ _ \ __/ _ \ '__|
+ *        | |\  |  __/ |_|  __/  __/ |  |  _| |  | |  __/ ||  __/ |
+ *        |_| \_|\___|\__|_|   \___|_|  |_| |_|  |_|\___|\__\___|_|
+ *
  *                  NetPerfMeter -- Network Performance Meter
- *                 Copyright (C) 2009-2024 by Thomas Dreibholz
+ *                 Copyright (C) 2009-2026 by Thomas Dreibholz
  * ==========================================================================
  *
  * This program is free software: you can redistribute it and/or modify
@@ -17,7 +23,7 @@
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
- * Contact:  thomas.dreibholz@gmail.com
+ * Contact:  dreibh@simula.no
  * Homepage: https://www.nntb.no/~dreibh/netperfmeter/
  */
 
@@ -33,7 +39,6 @@
 #include <math.h>
 
 #include <ctype.h>
-#include <sys/time.h>
 #include <sys/types.h>
 #include <netinet/tcp.h>
 #include <sys/socket.h>
@@ -50,9 +55,6 @@
 #include <iostream>
 
 
-Mutex gOutputMutex;
-
-
 // ###### Create formatted string (printf-like) #############################
 std::string format(const char* fmt, ...)
 {
@@ -61,17 +63,19 @@ std::string format(const char* fmt, ...)
    va_start(va, fmt);
    vsnprintf(buffer, sizeof(buffer), fmt, va);
    va_end(va);
-   return(std::string(buffer));
+   return std::string(buffer);
 }
 
 
 // ###### Get current time stamp ############################################
 unsigned long long getMicroTime()
 {
-  struct timeval tv;
-  gettimeofday(&tv,NULL);
-  return(((unsigned long long)tv.tv_sec * (unsigned long long)1000000) +
-         (unsigned long long)tv.tv_usec);
+  timespec ts;
+  if(__builtin_expect( (clock_gettime(CLOCK_REALTIME, &ts) != 0) , 0)) {
+     perror("clock_gettime():");
+     abort();
+  }
+  return ((unsigned long long)ts.tv_sec * 1000000ULL) + (ts.tv_nsec / 1000);
 }
 
 
@@ -101,31 +105,31 @@ int pollTimeout(const unsigned long long now, const size_t n, ...)
       timeout = std::min(timeout, t);
    }
    if(timeout == ~0ULL) {
-      return(-1);   // Infinite wait time (only care for sockets/files)
+      return -1;   // Infinite wait time (only care for sockets/files)
    }
    const double delta = (double)timeout - (double)now;
    if(delta <= 0.0) {
-      return(0);   // Do not wait, just check sockets/files
+      return 0;   // Do not wait, just check sockets/files
    }
    else {
       // Return wait time in milliseconds.
       // NOTE: Return ceiling of the value, since 999ULL/1000ULL == 0!
-      return((int)ceil(delta / 1000.0));
+      return (int)ceil(delta / 1000.0);
    }
 }
 
 
-/* ###### Length-checking strcpy() ###################################### */
+// ###### Length-checking strcpy() #########################################
 int safestrcpy(char* dest, const char* src, const size_t size)
 {
    assert(size > 0);
    strncpy(dest, src, size);
    dest[size - 1] = 0x00;
-   return(strlen(dest) < size);
+   return strlen(dest) < size;
 }
 
 
-/* ###### Length-checking strcat() ###################################### */
+// ###### Length-checking strcat() #########################################
 int safestrcat(char* dest, const char* src, const size_t size)
 {
    const size_t l1 = strlen(dest);
@@ -134,43 +138,43 @@ int safestrcat(char* dest, const char* src, const size_t size)
    assert(size > 0);
    strncat(dest, src, size - l1 - 1);
    dest[size - 1] = 0x00;
-   return(l1 + l2 < size);
+   return l1 + l2 < size;
 }
 
 
-/* ###### Find first occurrence of character in string ################### */
+// ###### Find first occurrence of character in string ######################
 static char* strindex(char* string, const char character)
 {
-   if(string != NULL) {
+   if(string != nullptr) {
       while(*string != character) {
          if(*string == 0x00) {
-            return(NULL);
+            return nullptr;
          }
          string++;
       }
-      return(string);
+      return string;
    }
-   return(NULL);
+   return nullptr;
 }
 
 
 
-/* ###### Find last occurrence of character in string #################### */
+// ###### Find last occurrence of character in string #######################
 static char* strrindex(char* string, const char character)
 {
    const char* original = string;
 
-   if(original != NULL) {
+   if(original != nullptr) {
       string = (char*)&string[strlen(string)];
       while(*string != character) {
          if(string == original) {
-            return(NULL);
+            return nullptr;
          }
          string--;
       }
-      return(string);
+      return string;
    }
-   return(NULL);
+   return nullptr;
 }
 
 
@@ -179,9 +183,9 @@ bool hasSuffix(const std::string& name, const std::string& suffix)
 {
    const size_t found = name.rfind(suffix);
    if(found == name.length() - suffix.length()) {
-      return(true);
+      return true;
    }
-   return(false);
+   return false;
 }
 
 
@@ -215,33 +219,33 @@ void dissectName(const std::string& name,
 }
 
 
-/* ###### Check for support of IPv6 ###################################### */
+// ###### Check for support of IPv6 #########################################
 bool checkIPv6()
 {
    int sd = socket(AF_INET6, SOCK_DGRAM, IPPROTO_UDP);
    if(sd >= 0) {
       close(sd);
-      return(true);
+      return true;
    }
-   return(false);
+   return false;
 }
 
 
-/* ###### Get socklen for given address ################################## */
+// ###### Get socklen for given address #####################################
 size_t getSocklen(const struct sockaddr* address)
 {
    switch(address->sa_family) {
       case AF_INET:
-         return(sizeof(struct sockaddr_in));
+         return sizeof(struct sockaddr_in);
       case AF_INET6:
-         return(sizeof(struct sockaddr_in6));
+         return sizeof(struct sockaddr_in6);
       default:
-         return(sizeof(struct sockaddr));
+         return sizeof(struct sockaddr);
    }
 }
 
 
-/* ###### Compare addresses ############################################## */
+// ###### Compare addresses #################################################
 int addresscmp(const struct sockaddr* a1, const struct sockaddr* a2, const bool port)
 {
    uint16_t     p1, p2;
@@ -273,26 +277,26 @@ int addresscmp(const struct sockaddr* a1, const struct sockaddr* a2, const bool
 
       result = memcmp((void*)&x1,(void*)&x2,16);
       if(result != 0) {
-         return(result);
+         return result;
       }
 
       if(port) {
          p1 = getPort((struct sockaddr*)a1);
          p2 = getPort((struct sockaddr*)a2);
          if(p1 < p2) {
-            return(-1);
+            return -1;
          }
          else if(p1 > p2) {
-            return(1);
+            return 1;
          }
       }
-      return(0);
+      return 0;
    }
-   return(0);
+   return 0;
 }
 
 
-/* ###### Convert address to string ###################################### */
+// ###### Convert address to string #########################################
 bool address2string(const struct sockaddr* address,
                     char*                  buffer,
                     const size_t           length,
@@ -316,7 +320,7 @@ bool address2string(const struct sockaddr* address,
          else {
             snprintf(buffer, length, "%s", inet_ntoa(ipv4address->sin_addr));
          }
-         return(true);
+         return true;
 
       case AF_INET6:
          ipv6address = (const struct sockaddr_in6*)address;
@@ -324,16 +328,16 @@ bool address2string(const struct sockaddr* address,
              (IN6_IS_ADDR_LINKLOCAL(&ipv6address->sin6_addr) ||
               IN6_IS_ADDR_MC_LINKLOCAL(&ipv6address->sin6_addr)) ) {
             ifname = if_indextoname(ipv6address->sin6_scope_id, (char*)&ifnamebuffer);
-            if(ifname == NULL) {
+            if(ifname == nullptr) {
                safestrcpy((char*)&ifnamebuffer, "(BAD!)", sizeof(ifnamebuffer));
-               return(false);
+               return false;
             }
             snprintf((char*)&scope, sizeof(scope), "%%%s", ifname);
          }
          else {
             scope[0] = 0x00;
          }
-         if(inet_ntop(AF_INET6, &ipv6address->sin6_addr, str, sizeof(str)) != NULL) {
+         if(inet_ntop(AF_INET6, &ipv6address->sin6_addr, str, sizeof(str)) != nullptr) {
             if(port) {
                snprintf(buffer, length,
                         "[%s%s]:%d", str, scope, ntohs(ipv6address->sin6_port));
@@ -341,19 +345,19 @@ bool address2string(const struct sockaddr* address,
             else {
                snprintf(buffer, length, "%s%s", str, scope);
             }
-            return(true);
+            return true;
          }
        break;
 
       case AF_UNSPEC:
          safestrcpy(buffer, "(unspecified)", length);
-         return(true);
+         return true;
    }
-   return(false);
+   return false;
 }
 
 
-/* ###### Convert string to address ###################################### */
+// ###### Convert string to address #########################################
 bool string2address(const char*           string,
                     union sockaddr_union* address,
                     const bool            readPort)
@@ -373,7 +377,7 @@ bool string2address(const char*           string,
    size_t i;
 
    if(strlen(string) > sizeof(host)) {
-      return(false);
+      return false;
    }
    strcpy((char*)&host,string);
    strcpy((char*)&port, "0");
@@ -381,7 +385,7 @@ bool string2address(const char*           string,
    /* ====== Handle RFC2732-compliant addresses ========================== */
    if(string[0] == '[') {
       p1 = strindex(host,']');
-      if(p1 != NULL) {
+      if(p1 != nullptr) {
          if((p1[1] == ':') || (p1[1] == '!')) {
             strcpy((char*)&port, &p1[2]);
          }
@@ -401,10 +405,10 @@ bool string2address(const char*           string,
          }
          if(colons == 1) {
             p1 = strrindex(host,':');
-            if(p1 == NULL) {
+            if(p1 == nullptr) {
                p1 = strrindex(host,'!');
             }
-            if(p1 != NULL) {
+            if(p1 != nullptr) {
                p1[0] = 0x00;
                strcpy((char*)&port, &p1[1]);
             }
@@ -417,14 +421,14 @@ bool string2address(const char*           string,
    if((sscanf(port, "%d", &portNumber) != 1) ||
       (portNumber < 0) ||
       (portNumber > 65535)) {
-      return(false);
+      return false;
    }
 
 
    /* ====== Create address structure ==================================== */
 
    /* ====== Get information for host ==================================== */
-   res        = NULL;
+   res        = nullptr;
    isNumeric  = true;
    isIPv6     = false;
    hostLength = strlen(host);
@@ -451,8 +455,8 @@ bool string2address(const char*           string,
       hints.ai_flags = AI_NUMERICHOST;
    }
 
-   if(getaddrinfo(host, NULL, &hints, &res) != 0) {
-      return(false);
+   if(getaddrinfo(host, nullptr, &hints, &res) != 0) {
+      return false;
    }
 
    memset((char*)address,0,sizeof(union sockaddr_union));
@@ -473,21 +477,21 @@ bool string2address(const char*           string,
        break;
 
       default:
-         return(false);
+         return false;
    }
 
    freeaddrinfo(res);
-   return(true);
+   return true;
 }
 
 
-/* ###### Print address ################################################## */
+// ###### Print address #####################################################
 void printAddress(std::ostream&          os,
                   const struct sockaddr* address,
                   const bool             port,
                   const bool             hideScope)
 {
-   static char str[128];
+   static char str[160];
 
    if(address2string(address, (char*)&str, sizeof(str), port, hideScope)) {
       os << str;
@@ -498,68 +502,78 @@ void printAddress(std::ostream&          os,
 }
 
 
-/* ###### Get protocol name ############################################## */
+// ###### Get protocol name #################################################
 const char* getProtocolName(const int protocol)
 {
-   const char* protocolName = "unknown?!";
+   const char* protocolName;
    switch(protocol) {
       case IPPROTO_SCTP:
          protocolName = "SCTP";
-         break;
+       break;
       case IPPROTO_TCP:
          protocolName = "TCP";
-         break;
-      case IPPROTO_MPTCP:
-         protocolName = "MPTCP";
-         break;
+       break;
       case IPPROTO_UDP:
          protocolName = "UDP";
-         break;
+       break;
+#ifdef HAVE_MPTCP
+      case IPPROTO_MPTCP:
+         protocolName = "MPTCP";
+       break;
+#endif
 #ifdef HAVE_DCCP
       case IPPROTO_DCCP:
          protocolName = "DCCP";
-         break;
+       break;
 #endif
+#ifdef HAVE_QUIC
+      case IPPROTO_QUIC:
+         protocolName = "QUIC";
+       break;
+#endif
+      default:
+         protocolName = "unknown?!";
+       break;
    }
-   return(protocolName);
+   return protocolName;
 }
 
 
-/* ###### Get port ####################################################### */
+// ###### Get port ##########################################################
 uint16_t getPort(const struct sockaddr* address)
 {
-   if(address != NULL) {
+   if(address != nullptr) {
       switch(address->sa_family) {
          case AF_INET:
-            return(ntohs(((struct sockaddr_in*)address)->sin_port));
+            return ntohs(((struct sockaddr_in*)address)->sin_port);
          case AF_INET6:
-            return(ntohs(((struct sockaddr_in6*)address)->sin6_port));
+            return ntohs(((struct sockaddr_in6*)address)->sin6_port);
          default:
-            return(0);
+            return 0;
       }
    }
-   return(0);
+   return 0;
 }
 
 
-/* ###### Set port ####################################################### */
+// ###### Set port ##########################################################
 bool setPort(struct sockaddr* address, uint16_t port)
 {
-   if(address != NULL) {
+   if(address != nullptr) {
       switch(address->sa_family) {
          case AF_INET:
             ((struct sockaddr_in*)address)->sin_port = htons(port);
-            return(true);
+            return true;
          case AF_INET6:
             ((struct sockaddr_in6*)address)->sin6_port = htons(port);
-            return(true);
+            return true;
       }
    }
-   return(false);
+   return false;
 }
 
 
-/* ###### Create socket ################################################## */
+// ###### Create socket #####################################################
 int createSocket(const int             family,
                  const int             type,
                  const int             protocol,
@@ -581,7 +595,6 @@ int createSocket(const int             family,
    int socketProtocol = protocol;
 #ifdef HAVE_MPTCP
    if(socketProtocol == IPPROTO_MPTCP) {
-      socketProtocol = IPPROTO_TCP;
       if(localAddresses > 1) {
          printf("WARNING: Currently, MPTCP does not support TCP_MULTIPATH_ADD. Binding to ANY address instead ...\n");
       }
@@ -591,13 +604,13 @@ int createSocket(const int             family,
    // ====== Create socket ==================================================
    int sd = ext_socket(socketFamily, type, socketProtocol);
    if(sd < 0) {
-      return(-2);
+      return -2;
    }
-   return(sd);
+   return sd;
 }
 
 
-/* ###### Bind socket #################################################### */
+// ###### Bind socket #######################################################
 int bindSocket(const int             sd,
                const int             family,
                const int             type,
@@ -637,7 +650,6 @@ int bindSocket(const int             sd,
    int socketProtocol = protocol;
 #ifdef HAVE_MPTCP
    if(socketProtocol == IPPROTO_MPTCP) {
-      socketProtocol = IPPROTO_TCP;
       if(localAddresses > 1) {
          printf("WARNING: Currently, MPTCP does not support TCP_MULTIPATH_ADD. Binding to ANY address instead ...\n");
          localAddressCount = 0;
@@ -651,32 +663,17 @@ int bindSocket(const int             sd,
       const int on = (bindV6Only == true) ? 1 : 0;
       // printf("IPV6_V6ONLY=%d\n", on);
       if(ext_setsockopt(sd, IPPROTO_IPV6, IPV6_V6ONLY, &on, sizeof(on)) < 0) {
-         return(-2);
+         return -2;
       }
    }
 #else
 #error IPV6_V6ONLY not defined?! Please create a bug report and provide some information about your OS!
-#endif
-// FIXME! Add proper, platform-independent code here!
-#ifndef __linux__
-#warning MPTCP is currently only available on Linux!
-#else
-   if((protocol == IPPROTO_MPTCP) || (protocol == IPPROTO_TCP)) {
-      const int cmtOnOff = (protocol == IPPROTO_MPTCP);
-      if(ext_setsockopt(sd, IPPROTO_TCP, MPTCP_ENABLED_LEGACY, &cmtOnOff, sizeof(cmtOnOff)) < 0) {
-         if(ext_setsockopt(sd, IPPROTO_TCP, MPTCP_ENABLED, &cmtOnOff, sizeof(cmtOnOff)) < 0) {
-            if(protocol == IPPROTO_MPTCP) {
-               return(-2);
-            }
-         }
-      }
-   }
 #endif
 
    // ====== Bind socket ====================================================
    if(localAddressCount == 0) {
       if(ext_bind(sd, &anyAddress.sa, getSocklen(&anyAddress.sa)) != 0) {
-         return(-3);
+         return -3;
       }
    }
    else {
@@ -701,7 +698,7 @@ int bindSocket(const int             sd,
          }
          if(sctp_bindx(sd, (sockaddr*)&buffer, localAddressCount,
                        SCTP_BINDX_ADD_ADDR) != 0) {
-            return(-3);
+            return -3;
          }
       }
       else {
@@ -720,7 +717,7 @@ int bindSocket(const int             sd,
          }
 
          if(ext_bind(sd, &localAddress.sa, getSocklen(&localAddress.sa)) != 0) {
-            return(-3);
+            return -3;
          }
       }
    }
@@ -729,42 +726,11 @@ int bindSocket(const int             sd,
    if(listenMode) {
       ext_listen(sd, 10);
    }
-   return(sd);
+   return sd;
 }
 
 
-/* ###### Create server socket of appropriate family and bind it ######### */
-int createAndBindSocket(const int             family,
-                        const int             type,
-                        const int             protocol,
-                        const uint16_t        localPort,
-                        const unsigned int    localAddresses,
-                        const sockaddr_union* localAddressArray,
-                        const bool            listenMode,
-                        const bool            bindV6Only)
-{
-   int sd = createSocket(family, type, protocol,
-                         localAddresses, localAddressArray);
-   if(sd >= 0) {
-      int reuse = 1;
-      if(ext_setsockopt(sd, SOL_SOCKET, SO_REUSEADDR, &reuse, sizeof(reuse)) < 0) {
-         std::cerr << "WARNING: Failed to configure socket reuse (SO_REUSEADDR option) - "
-                   << strerror(errno) << "!" << std::endl;
-      }
-
-      const int success = bindSocket(sd, family, type, protocol,
-                                     localPort, localAddresses, localAddressArray,
-                                     listenMode, bindV6Only);
-      if(success < 0) {
-         ext_close(sd);
-         return(success);
-      }
-   }
-   return(sd);
-}
-
-
-/* ###### Send SCTP ABORT ################################################ */
+// ###### Send SCTP ABORT ###################################################
 bool sendAbort(int sd, sctp_assoc_t assocID)
 {
    sctp_sndrcvinfo sinfo;
@@ -772,11 +738,11 @@ bool sendAbort(int sd, sctp_assoc_t assocID)
    sinfo.sinfo_assoc_id = assocID;
    sinfo.sinfo_flags    = SCTP_ABORT;
 
-   return(sctp_send(sd, NULL, 0, &sinfo, 0) >= 0);
+   return sctp_send(sd, nullptr, 0, &sinfo, 0) >= 0;
 }
 
 
-/* ###### Convert byte order of 64 bit value ############################# */
+// ###### Convert byte order of 64 bit value ################################
 static uint64_t byteswap64(const uint64_t value)
 {
 #if BYTE_ORDER == LITTLE_ENDIAN
@@ -787,24 +753,24 @@ static uint64_t byteswap64(const uint64_t value)
            ((int64_t)((b << 24) | ((b & 0x0000ff00) << 8) |
            ((b & 0x00ff0000) >> 8) | (b >> 24)) << 32) );
 #elif BYTE_ORDER == BIG_ENDIAN
-   return(value);
+   return value;
 #else
 #error Byte order undefined!
 #endif
 }
 
 
-/* ###### Convert byte order of 64 bit value ############################# */
+// ###### Convert byte order of 64 bit value ################################
 uint64_t hton64(const uint64_t value)
 {
-   return(byteswap64(value));
+   return byteswap64(value);
 }
 
 
-/* ###### Convert byte order of 64 bit value ############################# */
+// ###### Convert byte order of 64 bit value ################################
 uint64_t ntoh64(const uint64_t value)
 {
-   return(byteswap64(value));
+   return byteswap64(value);
 }
 
 
@@ -838,7 +804,7 @@ struct IeeeDouble {
 };
 
 
-/* ###### Convert double to machine-independent form ##################### */
+// ###### Convert double to machine-independent form ########################
 network_double_t doubleToNetwork(const double d)
 {
    struct IeeeDouble ieee;
@@ -887,11 +853,11 @@ network_double_t doubleToNetwork(const double d)
       ieee.f1 = (unsigned long)ldexp (frac, DBL_FRC1_BITS);
       ieee.f2 = (unsigned long)ldexp (frac, DBL_FRC_BITS);
    }
-   return(hton64(*((network_double_t*)&ieee)));
+   return hton64(*((network_double_t*)&ieee));
 }
 
 
-/* ###### Convert machine-independent form to double ##################### */
+// ###### Convert machine-independent form to double ########################
 double networkToDouble(network_double_t value)
 {
    network_double_t   hValue;
@@ -929,7 +895,7 @@ double networkToDouble(network_double_t value)
          d = -d;
       }
    }
-   return(d);
+   return d;
 }
 
 #else
@@ -940,20 +906,20 @@ union DoubleIntUnion
    unsigned long long Integer;
 };
 
-/* ###### Convert double to machine-independent form ##################### */
+// ###### Convert double to machine-independent form ########################
 network_double_t doubleToNetwork(const double d)
 {
    union DoubleIntUnion valueUnion;
    valueUnion.Double = d;
-   return(hton64(valueUnion.Integer));
+   return hton64(valueUnion.Integer);
 }
 
-/* ###### Convert machine-independent form to double ##################### */
+// ###### Convert machine-independent form to double ########################
 double networkToDouble(network_double_t value)
 {
    union DoubleIntUnion valueUnion;
    valueUnion.Integer = ntoh64(value);
-   return(valueUnion.Double);
+   return valueUnion.Double;
 }
 
 #endif
@@ -966,7 +932,7 @@ double networkToDouble(network_double_t value)
 #define KILL_TIMEOUT 2000000
 
 
-/* ###### Global variables ############################################### */
+// ###### Global variables ##################################################
 static bool   DetectedBreak = false;
 static bool   PrintedBreak  = false;
 static bool   Quiet         = false;
@@ -977,7 +943,7 @@ static unsigned long long LastDetection = (unsigned long long)-1;
 #endif
 
 
-/* ###### Handler for SIGINT ############################################# */
+// ###### Handler for SIGINT ################################################
 void breakDetector(int signum)
 {
    DetectedBreak = true;
@@ -998,7 +964,7 @@ void breakDetector(int signum)
 }
 
 
-/* ###### Install break detector ######################################### */
+// ###### Install break detector ############################################
 void installBreakDetector()
 {
    DetectedBreak = false;
@@ -1013,7 +979,7 @@ void installBreakDetector()
 }
 
 
-/* ###### Unnstall break detector ######################################## */
+// ###### Unnstall break detector ###########################################
 void uninstallBreakDetector()
 {
    signal(SIGINT,SIG_DFL);
@@ -1028,7 +994,7 @@ void uninstallBreakDetector()
 }
 
 
-/* ###### Check, if break has been detected ############################## */
+// ###### Check, if break has been detected #################################
 bool breakDetected()
 {
    if((DetectedBreak) && (!PrintedBreak)) {
@@ -1037,11 +1003,11 @@ bool breakDetected()
       }
       PrintedBreak = getMicroTime();
    }
-   return(DetectedBreak);
+   return DetectedBreak;
 }
 
 
-/* ###### Send break to main thread ###################################### */
+// ###### Send break to main thread #########################################
 void sendBreak(const bool quiet)
 {
    Quiet = quiet;
@@ -1050,7 +1016,7 @@ void sendBreak(const bool quiet)
 
 
 
-/* ###### Get random value using specified random number generator ####### */
+// ###### Get random value using specified random number generator ##########
 double getRandomValue(const double* valueArray, const uint8_t rng)
 {
    double value;
@@ -1058,40 +1024,43 @@ double getRandomValue(const double* valueArray, const uint8_t rng)
       case RANDOM_CONSTANT:
          value = valueArray[0];
        break;
+      case RANDOM_UNIFORM:
+         value = valueArray[0] + (randomDouble() * (valueArray[1] - valueArray[0]));
+       break;
       case RANDOM_EXPONENTIAL:
          value = randomExpDouble(valueArray[0]);
        break;
-      case RANDOM_UNIFORM: {
-         const double lowest  = valueArray[0] - valueArray[1]*valueArray[0];
-         const double highest = valueArray[0] + valueArray[1]*valueArray[0];
-         value = lowest + (randomDouble() * (highest - lowest));
-        }
-       break;
       case RANDOM_PARETO:
          value = randomParetoDouble(valueArray[0], valueArray[1]);
        break;
+      case RANDOM_NORMAL:
+         value = randomNormal(valueArray[0], valueArray[1]);
+       break;
+      case RANDOM_TRUNCATED_NORMAL:
+         value = randomTruncNormal(valueArray[0], valueArray[1]);
+       break;
       default:
-         assert(false);
+         abort();
        break;
    }
-   return(value);
+   return value;
 }
 
 
-/* ###### Get name of specified random number generator ################## */
+// ###### Get name of specified random number generator #####################
 const char* getRandomGeneratorName(const uint8_t rng)
 {
    switch(rng) {
       case RANDOM_CONSTANT:
-         return("constant");
-      case RANDOM_EXPONENTIAL:
-         return("exponential");
+         return "constant";
       case RANDOM_UNIFORM:
-         return("uniform");
+         return "uniform";
+      case RANDOM_EXPONENTIAL:
+         return "exponential";
       case RANDOM_PARETO:
-         return("pareto");
+         return "pareto";
    }
-   return("(invalid!)");
+   return "(invalid!)";
 }
 
 
@@ -1108,53 +1077,53 @@ const char* getRandomGeneratorName(const uint8_t rng)
 #define RS_CLIB       2
 
 static int   RandomSource = RS_TRY_DEVICE;
-static FILE* RandomDevice = NULL;
+static FILE* RandomDevice = nullptr;
 
 
-/* ###### Get 8-bit random value ######################################### */
+// ###### Get 8-bit random value ############################################
 uint8_t random8()
 {
-   return((uint8_t)random32());
+   return (uint8_t)random32();
 }
 
 
-/* ###### Get 16-bit random value ######################################## */
+// ###### Get 16-bit random value ###########################################
 uint16_t random16()
 {
-   return((uint16_t)random32());
+   return (uint16_t)random32();
 }
 
 
-/* ###### Get 64-bit random value ######################################## */
+// ###### Get 64-bit random value ###########################################
 uint64_t random64()
 {
-   return( (((uint64_t)random32()) << 32) | (uint64_t)random32() );
+   return (((uint64_t)random32()) << 32) | (uint64_t)random32();
 }
 
 
-/* ###### Get 32-bit random value ######################################## */
+// ###### Get 32-bit random value ###########################################
 uint32_t random32()
 {
 #if defined(SIM_IMPORT) || defined(OMNETPPLIBS_IMPORT)
 #warning Using OMNeT++ random generator instead of time-seeded one!
    const double value = uniform(0.0, (double)0xffffffff);
-   return((uint32_t)rint(value));
+   return (uint32_t)rint(value);
 #else
    uint32_t number;
 
    switch(RandomSource) {
       case RS_DEVICE:
          if(fread(&number, sizeof(number), 1, RandomDevice) == 1) {
-            return(number);
+            return number;
          }
          RandomSource = RS_CLIB;
       case RS_TRY_DEVICE:
          RandomDevice = fopen("/dev/urandom", "r");
-         if(RandomDevice != NULL) {
+         if(RandomDevice != nullptr) {
             if(fread(&number, sizeof(number), 1, RandomDevice) == 1) {
                srandom(number);
                RandomSource = RS_DEVICE;
-               return(number);
+               return number;
             }
             fclose(RandomDevice);
          }
@@ -1164,26 +1133,49 @@ uint32_t random32()
    }
    const uint16_t a = random() & 0xffff;
    const uint16_t b = random() & 0xffff;
-   return( (((uint32_t)a) << 16) | (uint32_t)b );
+   return (((uint32_t)a) << 16) | (uint32_t)b;
 #endif
 }
 
 
-/* ###### Get double random value ######################################## */
+// ###### Get double random value ###########################################
 double randomDouble()
 {
-   return( (double)random32() / (double)4294967296.0 );
+   // Resulting range: [0, 1)
+   return (double)random32() / (double)0x100000000;
 }
 
 
-/* ###### Get exponential-distributed double random value ################ */
-double randomExpDouble(const double p)
+// ###### Get exponential-distributed double random value ###################
+double randomExpDouble(const double mean)
 {
-   return( -p * log(randomDouble()) );
+   // randomDouble() returns value in [0, 1), i.e. 0 is included. log(0) = -∞.
+   // => Using 1.0 - randomDouble() to prevent this issue:
+   return -mean * log(1.0 - randomDouble());
 }
 
 
-/* ###### Get pareto-distributed double random value ##################### */
+// ###### Get normal-distributed double random value ########################
+double randomNormal(const double mean, const double stddev)
+{
+    const double u = randomDouble();
+    const double v = randomDouble();
+    return mean + stddev * sqrt(-2.0*log(u)) * cos(M_PI*2.0*v);
+}
+
+
+// ###### Get truncated-normal-distributed double random value ##############
+double randomTruncNormal(const double mean, const double stddev)
+{
+   double result;
+   do {
+       result = randomNormal(mean, stddev);
+   } while(result < 0);
+   return result;
+}
+
+
+// ###### Get pareto-distributed double random value ########################
 // Parameters:
 // location = the location parameter (also: scale parameter): x_m, x_min or m
 // shape    = the shape parameter: alpha or k
@@ -1212,11 +1204,11 @@ double randomParetoDouble(const double location, const double shape)
    while ((r <= 0.0) || (r >= 1.0)) {
       r = randomDouble();
    }
-   return( location / pow(r, 1.0 / shape) );
+   return location / pow(r, 1.0 / shape);
 }
 
 
-/* ###### poll() to select() wrapper to work around broken Apple poll() ## */
+// ###### poll() to select() wrapper to work around broken Apple poll() #####
 #ifdef __APPLE__
 #warning Using poll() to select() wrapper to work around broken Apple poll().
 int ext_poll_wrapper(struct pollfd* fdlist, long unsigned int count, int time)
@@ -1230,7 +1222,7 @@ int ext_poll_wrapper(struct pollfd* fdlist, long unsigned int count, int time)
    unsigned int i;
 
    if(time < 0)
-      to = NULL;
+      to = nullptr;
    else {
       to = &timeout;
       timeout.tv_sec  = time / 1000;
@@ -1266,7 +1258,7 @@ int ext_poll_wrapper(struct pollfd* fdlist, long unsigned int count, int time)
    // ====== Do ext_select() ================================================
    result = ext_select(n + 1, &readfdset, &writefdset ,&exceptfdset, to);
    if(result < 0) {
-      return(result);
+      return result;
    }
 
    // ====== Set result flags ===============================================
@@ -1281,87 +1273,6 @@ int ext_poll_wrapper(struct pollfd* fdlist, long unsigned int count, int time)
          fdlist[i].revents |= POLLERR;
       }
    }
-   return(result);
+   return result;
 }
 #endif
-
-
-#if 0
-/* ###### Word-around for lksctp bug ##################################### */
-ssize_t sctp_send_fixed(int                           sd,
-                        const void*                   data,
-                        size_t                        len,
-                        const struct sctp_sndrcvinfo* sinfo,
-                        int                           flags)
-{
-   struct sctp_sndrcvinfo* sri;
-   struct iovec            iov = { (char*)data, len };
-   struct cmsghdr*         cmsg;
-   socklen_t               cmsglen = CMSG_SPACE(sizeof(struct sctp_sndrcvinfo));
-   char                    cbuf[CMSG_SPACE(sizeof(struct sctp_sndrcvinfo))];
-   struct msghdr msg = {
-      NULL, 0,
-      &iov, 1,
-      cbuf, cmsglen,
-      flags
-   };
-
-   cmsg = (struct cmsghdr*)CMSG_FIRSTHDR(&msg);
-   cmsg->cmsg_len   = CMSG_LEN(sizeof(struct sctp_sndrcvinfo));
-   cmsg->cmsg_level = IPPROTO_SCTP;
-   cmsg->cmsg_type  = SCTP_SNDRCV;
-
-   sri = (struct sctp_sndrcvinfo*)CMSG_DATA(cmsg);
-   memcpy(sri, sinfo, sizeof(struct sctp_sndrcvinfo));
-   return(ext_sendmsg(sd, &msg, msg.msg_flags));
-}
-#endif
-
-
-/* ###### Configure send and receive buffer sizes ######################## */
-bool setBufferSizes(int sd, const int sndBufSize, const int rcvBufSize)
-{
-   if(sndBufSize > 0) {
-      if(ext_setsockopt(sd, SOL_SOCKET, SO_SNDBUF, &sndBufSize, sizeof(sndBufSize)) < 0) {
-         std::cerr << "ERROR: Failed to configure send buffer size (SO_SNDBUF option) - "
-                   << strerror(errno) << "!" << std::endl;
-         return(false);
-      }
-      int newBufferSize = 0;
-      socklen_t newBufferSizeLength = sizeof(newBufferSize);
-      if(ext_getsockopt(sd, SOL_SOCKET, SO_SNDBUF, &newBufferSize, &newBufferSizeLength) < 0) {
-         std::cerr << "ERROR: Failed to obtain receive send size (SO_SNDBUF option) - "
-                   << strerror(errno) << "!" << std::endl;
-         return(false);
-      }
-      // printf("SET-SNDBUF: sd=%d - %d (requested %d)\n", sd, newBufferSize, sndBufSize);
-      if(newBufferSize < sndBufSize) {
-         std::cerr << "ERROR: actual send buffer size < configured send buffer size: "
-                   << newBufferSize << " < " << sndBufSize
-                   << std::endl;
-         return(false);
-      }
-   }
-   if(rcvBufSize > 0) {
-      if(ext_setsockopt(sd, SOL_SOCKET, SO_RCVBUF, &rcvBufSize, sizeof(rcvBufSize)) < 0) {
-         std::cerr << "ERROR: Failed to configure receive buffer size (SO_RCVBUF option) - "
-                   << strerror(errno) << "!" << std::endl;
-         return(false);
-      }
-      int newBufferSize = 0;
-      socklen_t newBufferSizeLength = sizeof(newBufferSize);
-      if(ext_getsockopt(sd, SOL_SOCKET, SO_RCVBUF, &newBufferSize, &newBufferSizeLength) < 0) {
-         std::cerr << "ERROR: Failed to obtain receive buffer size (SO_RCVBUF option) - "
-                   << strerror(errno) << "!" << std::endl;
-         return(false);
-      }
-      // printf("SET-RCVBUF: sd=%d - %d (requested %d)\n", sd, newBufferSize, rcvBufSize);
-      if(newBufferSize < rcvBufSize) {
-         std::cerr << "ERROR: actual receive buffer size < configured receive buffer size: "
-                   << newBufferSize << " < " << rcvBufSize
-                   << std::endl;
-         return(false);
-      }
-   }
-   return(true);
-}
