@@ -1,7 +1,13 @@
 /*
  * ==========================================================================
+ *         _   _      _   ____            __ __  __      _
+ *        | \ | | ___| |_|  _ \ ___ _ __ / _|  \/  | ___| |_ ___ _ __
+ *        |  \| |/ _ \ __| |_) / _ \ '__| |_| |\/| |/ _ \ __/ _ \ '__|
+ *        | |\  |  __/ |_|  __/  __/ |  |  _| |  | |  __/ ||  __/ |
+ *        |_| \_|\___|\__|_|   \___|_|  |_| |_|  |_|\___|\__\___|_|
+ *
  *                  NetPerfMeter -- Network Performance Meter
- *                 Copyright (C) 2009-2024 by Thomas Dreibholz
+ *                 Copyright (C) 2009-2026 by Thomas Dreibholz
  * ==========================================================================
  *
  * This program is free software: you can redistribute it and/or modify
@@ -17,19 +23,19 @@
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
- * Contact:  thomas.dreibholz@gmail.com
+ * Contact:  dreibh@simula.no
  * Homepage: https://www.nntb.no/~dreibh/netperfmeter/
  */
 
-#include "transfer.h"
 #include "control.h"
+#include "loglevel.h"
 #include "tools.h"
-#include "netperfmeterpackets.h"
 
-#include <string.h>
-#include <assert.h>
-#include <math.h>
-#include <iostream>
+#include <cstring>
+
+
+#define MAXIMUM_MESSAGE_SIZE (size_t)65536
+#define MAXIMUM_PAYLOAD_SIZE (MAXIMUM_MESSAGE_SIZE - sizeof(NetPerfMeterDataMessage))
 
 
 static void updateStatistics(Flow*                          flowSpec,
@@ -37,10 +43,6 @@ static void updateStatistics(Flow*                          flowSpec,
                              const NetPerfMeterDataMessage* dataMsg,
                              const size_t                   received);
 
-extern unsigned int gOutputVerbosity;
-
-#define MAXIMUM_MESSAGE_SIZE (size_t)65536
-#define MAXIMUM_PAYLOAD_SIZE (MAXIMUM_MESSAGE_SIZE - sizeof(NetPerfMeterDataMessage))
 
 
 // ###### Generate payload pattern ##########################################
@@ -122,20 +124,12 @@ ssize_t sendNetPerfMeterData(Flow*                    flow,
          const bool sendUnreliable = (randomDouble() > flow->getTrafficSpec().ReliableMode);
          if(sendUnreliable) {
             sinfo.sinfo_timetolive = flow->getTrafficSpec().RetransmissionTrials;
-#if defined __FreeBSD__ || defined __APPLE__
-            /* This is implementation specific and should be changed
-             * to use either a SCTP_PRINFO cmsg or sctp_sendv(). However,
-             * these are currently also only available in FreeBSD and Mac OS X
-             */
             if(flow->getTrafficSpec().RetransmissionTrialsInMS) {
                sinfo.sinfo_flags |= SCTP_PR_SCTP_TTL;
             }
             else {
                sinfo.sinfo_flags |= SCTP_PR_SCTP_RTX;
             }
-#else
-#warning The SCTP API does not support multiple PR-SCTP policies! Using SCTP_PR_SCTP_TTL in unreliable mode.
-#endif
          }
       }
       if(flow->getTrafficSpec().OrderedMode < 1.0) {
@@ -148,6 +142,21 @@ ssize_t sendNetPerfMeterData(Flow*                    flow,
                        (char*)&outputBuffer, bytesToSend,
                        &sinfo, 0);
    }
+#ifdef HAVE_QUIC
+   else if(flow->getTrafficSpec().Protocol == IPPROTO_QUIC) {
+      int64_t  sid;
+      uint32_t flags;
+      if(flow->isAcceptedIncomingFlow()) {   // from passive side (server)
+         sid   = ((int64_t)flow->getStreamID() << 2) | QUIC_STREAM_TYPE_SERVER_MASK | QUIC_STREAM_TYPE_UNI_MASK;
+         flags = (flow->getFirstTransmission() == 0) ? MSG_QUIC_STREAM_NEW : 0;
+      }
+      else {
+         sid   = ((int64_t)flow->getStreamID() << 2) | QUIC_STREAM_TYPE_UNI_MASK;
+         flags = 0;   // already created by Identify procedure!
+      }
+      sent = quic_sendmsg(flow->getSocketDescriptor(), (char*)&outputBuffer, bytesToSend, sid, flags);
+   }
+#endif
    else if(flow->getTrafficSpec().Protocol == IPPROTO_UDP) {
       if(flow->isRemoteAddressValid()) {
          sent = ext_sendto(flow->getSocketDescriptor(),
@@ -170,133 +179,193 @@ ssize_t sendNetPerfMeterData(Flow*                    flow,
       (!flow->isAcceptedIncomingFlow()) &&
       (flow->getTrafficSpec().ErrorOnAbort) &&
       (flow->getOutputStatus() == Flow::On)) {
-      gOutputMutex.lock();
-      std::cerr << "ERROR: Flow #" << flow->getFlowID() << " has been aborted - "
-                << strerror(errno) << "!" << std::endl;
-      gOutputMutex.unlock();
-      exit(1);
+      LOG_FATAL
+      stdlog << format("Flow #%u on socket %d has been aborted: %s!",
+                       flow->getFlowID(), flow->getSocketDescriptor(),
+                       strerror(errno)) << "\n";
+      LOG_END_FATAL
    }
 
-   return(sent);
+   return sent;
 }
 
 
 // ###### Transmit data frame ###############################################
-ssize_t transmitFrame(Flow*                    flow,
-                      const unsigned long long now)
+bool transmitFrame(Flow*                    flow,
+                   const unsigned long long now)
 {
    // ====== Obtain length of data to send ==================================
-   size_t bytesToSend =
-      (size_t)rint(getRandomValue((const double*)&flow->getTrafficSpec().OutboundFrameSize,
-                                  flow->getTrafficSpec().OutboundFrameSizeRng));
-   if(bytesToSend == 0) {
-      // On POLLOUT, we generate a maximum-sized message. If there is still space
-      // in the buffer, POLLOUT will be set again ...
-      bytesToSend = std::min((size_t)flow->getTrafficSpec().MaxMsgSize, MAXIMUM_MESSAGE_SIZE);
-   }
+   ssize_t bytesToSend =
+      (ssize_t)rint(getRandomValue((const double*)&flow->getTrafficSpec().OutboundFrameSize,
+                                   flow->getTrafficSpec().OutboundFrameSizeRng));
    ssize_t bytesSent   = 0;
    size_t  packetsSent = 0;
+   if(bytesToSend > 0) {
+      const uint32_t frameID = flow->nextOutboundFrameID();
+      while(bytesSent < (ssize_t)bytesToSend) {
+         // ====== Send message =============================================
+         ssize_t chunkSize = std::min(bytesToSend - bytesSent,
+                                      std::min((ssize_t)flow->getTrafficSpec().MaxMsgSize,
+                                               (ssize_t)MAXIMUM_MESSAGE_SIZE));
+         const ssize_t sent =
+            sendNetPerfMeterData(flow, frameID,
+                                 (bytesSent == 0),                       // Is frame begin?
+                                 (bytesSent + chunkSize >= bytesToSend), // Is frame end?
+                                 now, chunkSize);
+         // NOTE: Due to minimum size for a NETPERFMETER_DATA chunk (48 B),
+         //       the sent size may be >= chunkSize!
 
-   const uint32_t frameID = flow->nextOutboundFrameID();
-   while(bytesSent < (ssize_t)bytesToSend) {
-      // ====== Send message ================================================
-      size_t chunkSize = std::min(bytesToSend, std::min((size_t)flow->getTrafficSpec().MaxMsgSize,
-                                                        MAXIMUM_MESSAGE_SIZE));
-      const ssize_t sent =
-         sendNetPerfMeterData(flow, frameID,
-                              (bytesSent == 0),                       // Is frame begin?
-                              (bytesSent + chunkSize >= bytesToSend), // Is frame end?
-                              now, chunkSize);
-
-      // ====== Update statistics ===========================================
-      if(sent > 0) {
-         bytesSent += sent;
-         packetsSent++;
-      }
-      else {
-         // Transmission error -> stop sending.
-         break;
+         // ====== Update statistics ========================================
+         if(sent > 0) {
+            bytesSent += sent;
+            packetsSent++;
+         }
+         else {
+            // Transmission error -> stop sending.
+            break;
+         }
       }
    }
+   else {
+      bytesToSend = 0;   // There is nothing to send
+   }
 
+   // ====== Update statistics ==============================================
    flow->updateTransmissionStatistics(now, 1, packetsSent, bytesSent);
-   return(bytesSent);
+   return (bytesSent >= bytesToSend);
 }
 
 
 // ###### Handle data message ###############################################
-ssize_t handleNetPerfMeterData(const bool               isActiveMode,
-                               const unsigned long long now,
-                               const int                protocol,
-                               const int                sd)
+bool handleNetPerfMeterData(const bool               isActiveMode,
+                            const unsigned long long now,
+                            const int                protocol,
+                            const int                sd)
 {
    char            inputBuffer[65536];
    sockaddr_union  from;
-   socklen_t       fromlen = sizeof(from);
-   int             flags   = 0;
-   sctp_sndrcvinfo sinfo;
+   socklen_t       fromlen  = sizeof(from);
+   int             flags    = 0;
+   int64_t         streamID = 0;
 
-   sinfo.sinfo_stream = 0;
+   // ====== Read message (or fragment) =====================================
    const ssize_t received =
       FlowManager::getFlowManager()->getMessageReader()->receiveMessage(
-         sd, &inputBuffer, sizeof(inputBuffer), &from.sa, &fromlen, &sinfo, &flags);
+         sd, &inputBuffer, sizeof(inputBuffer), &from.sa, &fromlen, &streamID, &flags);
+   if(received == MRRM_PARTIAL_READ) {
+      return true;   // Partial read -> wait for next fragment.
+   }
 
-   if( (received > 0) && (!(flags & MSG_NOTIFICATION)) ) {
-      const NetPerfMeterDataMessage*     dataMsg     =
-         (const NetPerfMeterDataMessage*)&inputBuffer;
-      const NetPerfMeterIdentifyMessage* identifyMsg =
-         (const NetPerfMeterIdentifyMessage*)&inputBuffer;
+   // ====== Handle data ====================================================
+   if(received > 0) {
+      if(!(flags & MSG_NOTIFICATION)) {
+         const NetPerfMeterDataMessage*     dataMsg     =
+            (const NetPerfMeterDataMessage*)&inputBuffer;
+         const NetPerfMeterIdentifyMessage* identifyMsg =
+            (const NetPerfMeterIdentifyMessage*)&inputBuffer;
 
-      // ====== Handle NETPERFMETER_IDENTIFY_FLOW message ===================
-      if( (received >= (ssize_t)sizeof(NetPerfMeterIdentifyMessage)) &&
-          (identifyMsg->Header.Type == NETPERFMETER_IDENTIFY_FLOW) &&
-          (ntoh64(identifyMsg->MagicNumber) == NETPERFMETER_IDENTIFY_FLOW_MAGIC_NUMBER) ) {
-          handleNetPerfMeterIdentify(identifyMsg, sd, &from);
+         // ====== Handle NETPERFMETER_IDENTIFY_FLOW message ================
+         if( (received >= (ssize_t)sizeof(NetPerfMeterIdentifyMessage)) &&
+            (identifyMsg->Header.Type == NETPERFMETER_IDENTIFY_FLOW) &&
+            (ntoh64(identifyMsg->MagicNumber) == NETPERFMETER_IDENTIFY_FLOW_MAGIC_NUMBER) ) {
+            const bool identifyOkay =
+               handleNetPerfMeterIdentify(identifyMsg, sd, &from);
+            if(!identifyOkay) {
+               LOG_WARNING
+               stdlog << format("Failed handling NETPERFMETER_IDENTIFY on socket %d!", sd) << "\n";
+               LOG_END
+               if(protocol != IPPROTO_UDP) {
+                  ext_shutdown(sd, SHUT_RDWR);
+               }
+               return false;
+            }
+         }
+
+         // ====== Handle NETPERFMETER_DATA message =========================
+         else if( (received >= (ssize_t)sizeof(NetPerfMeterDataMessage)) &&
+                  (dataMsg->Header.Type == NETPERFMETER_DATA) ) {
+            // ====== Identify flow =========================================
+            Flow* flow;
+            if( (protocol == IPPROTO_UDP) && (!isActiveMode) ) {
+               flow = FlowManager::getFlowManager()->findFlow(&from.sa);
+            }
+            else if(protocol == IPPROTO_SCTP) {
+               // Flow ID = SCTP stream ID:
+               flow = FlowManager::getFlowManager()->findFlow(sd, (uint32_t)streamID);
+            }
+#ifdef HAVE_QUIC
+            else if(protocol == IPPROTO_QUIC) {
+               // Flow ID = QUIC stream ID (remove 2 last bits:
+               const uint32_t flowID = (uint32_t)(streamID >> 2);
+               flow = FlowManager::getFlowManager()->findFlow(sd, flowID);
+            }
+#endif
+            else {
+               // Flow ID is always 0:
+               flow = FlowManager::getFlowManager()->findFlow(sd, 0);
+            }
+            if(flow) {
+               // Update flow statistics by received NETPERFMETER_DATA message.
+               updateStatistics(flow, now, dataMsg, received);
+            }
+            else {
+               LOG_WARNING
+               stdlog << format("Received NETPERFMETER_DATA for unknown flow on socket %d!", sd) << "\n";
+               LOG_END
+               if(protocol != IPPROTO_UDP) {
+                  ext_shutdown(sd, SHUT_RDWR);
+               }
+               return false;
+            }
+         }
+         else {
+            LOG_WARNING
+            stdlog << format("Received garbage on socket %d!", sd) << "\n";
+            LOG_END
+            if(protocol != IPPROTO_UDP) {
+               ext_shutdown(sd, SHUT_RDWR);
+            }
+            return false;
+         }
       }
+   }
 
-      // ====== Handle NETPERFMETER_DATA message ============================
-      else if( (received >= (ssize_t)sizeof(NetPerfMeterDataMessage)) &&
-               (dataMsg->Header.Type == NETPERFMETER_DATA) ) {
-         // ====== Identify flow ============================================
-         Flow* flow;
-         if(( protocol == IPPROTO_UDP) && (!isActiveMode) ) {
-            flow = FlowManager::getFlowManager()->findFlow(&from.sa);
+   // ====== Handle error ===================================================
+   else {
+      Flow* flow = FlowManager::getFlowManager()->findFlow(sd, 0);
+      if(flow) {
+         flow->lock();
+         if(!flow->isAcceptedIncomingFlow()) {
+            // The outgoing flow on the active side is closed:
+            LOG_WARNING
+            stdlog << format("End of input for flow #%u on socket %d!",
+                             flow->getFlowID(), sd) << "\n";
+            LOG_END
          }
          else {
-            flow = FlowManager::getFlowManager()->findFlow(sd, sinfo.sinfo_stream);
-         }
-         if(flow) {
-            // Update flow statistics by received NETPERFMETER_DATA message.
-            updateStatistics(flow, now, dataMsg, received);
-         }
-         else {
-            gOutputMutex.lock();
-            std::cout << "WARNING: Received data for unknown flow!" << std::endl;
-            gOutputMutex.unlock();
+            // This is probably just the regular connection shutdown:
+            LOG_DEBUG
+            stdlog << format("End of input for flow #%u on socket %d!",
+                             flow->getFlowID(), sd) << "\n";
+            LOG_END
          }
+         flow->unlock();
+         flow->endOfInput();
+         return flow->isAcceptedIncomingFlow();   // No error for incoming flow!
       }
       else {
-         gOutputMutex.lock();
-         std::cout << "WARNING: Received garbage!" << std::endl;
-         gOutputMutex.unlock();
+         LOG_WARNING
+         stdlog << format("End of input for unidentified flow on socket %d!",
+                          sd) << "\n";
+         LOG_END
       }
+      if(protocol != IPPROTO_UDP) {
+         ext_shutdown(sd, SHUT_RDWR);
+      }
+      return false;
    }
 
-   else if( (received <= 0) && (received != MRRM_PARTIAL_READ) ) {
-      Flow* flow = FlowManager::getFlowManager()->findFlow(sd, sinfo.sinfo_stream);
-      if(flow) {
-         if(gOutputVerbosity >= NPFOV_CONNECTIONS) {
-            flow->lock();
-            gOutputMutex.lock();
-            std::cout << "End of input for flow " <<  flow->getFlowID() << std::endl;
-            gOutputMutex.unlock();
-            flow->unlock();
-         }
-         flow->endOfInput();
-      }
-   }
-
-   return(received);
+   return true;
 }
 
 
